//*****************************************************************************
//              \ /
// No Copyright (c) 2003
//              / \
// No rights reserved. This material contains published,
// non-copyrighted work, which includes non-confidential and
// non-proprietary information of Kurt Santel.
//
// Source: CodeWarCommands.h
//
// Description: Common header file with the commands that are sent between
//              the client and the server
//
//*****************************************************************************

#pragma once

#include <utility>
#include "CodeWarTypes.h"

////////////////////////////////////////////////////////////////////
// Info that commands or events can contain

struct StatusInfo
{
    StatusInfo(FoodType f = -1, LifeType l = -1, HeadingType h = North)
    {
        m_food = f;
        m_life = l;
        m_heading = h;
    }

    FoodType m_food;
    LifeType m_life;
    HeadingType m_heading;
};

////////////////////////////////////////////////////////////////////
// The base struct (this will not be sent)

struct BasicCommand
{
    BasicCommand()
    {
        m_version = theVersion;
        m_command = cmdSkipTurn;
    }

    VersionType m_version;
    CommandType m_command;
};

////////////////////////////////////////////////////////////////////
// The commands Server -> Client

struct pollCommand : BasicCommand
{
    pollCommand()
    {
        m_command = cmdPoll;
    }
};

struct infoCommand : BasicCommand
{
    infoCommand(IDType id=-1, StatusInfo s = StatusInfo())
    {
        m_command = cmdInfo;
        m_id = id;
        m_status = s;
    }

    IDType m_id;
    StatusInfo m_status;
};

//***********************************************************
// This is sent every time a client connects and disconnects
// NOT YET IMPLEMENTED
//***********************************************************
struct generalInfoCommand : BasicCommand
{
    generalInfoCommand(NrType Nr = -1)
    {
        m_command = cmdGeneralInfo;
        m_Nr = Nr;
    }

    NrType m_Nr;
};

struct onDeadCommand : BasicCommand
{
    onDeadCommand()
    {
        m_command = cmdDead;
    }
};

struct onHitWallCommand : BasicCommand
{
    onHitWallCommand()
    {
        m_command = cmdHitWall;
    }
};

struct onRobotInSightCommand : BasicCommand
{
    onRobotInSightCommand()
    {
        m_command = cmdRobotInSight;
    }
};

struct onFoodFoundCommand : BasicCommand
{
    onFoodFoundCommand()
    {
        m_command = cmdFoodFound;
    }
};

struct onAttackedCommand : BasicCommand
{
    onAttackedCommand()
    {
        m_command = cmdAttacked;
    }
};

////////////////////////////////////////////////////////////////////
// The commands Client -> Server

struct skipTurnCommand : BasicCommand
{
    skipTurnCommand()
    {
        m_command = cmdSkipTurn;
    }
};

struct registerCommand : BasicCommand
{
    registerCommand(AttackType a = -1, DefenseType d = -1)
    {
        m_command = cmdRegister;
        m_attack = a;
        m_defense = d;
    }

    AttackType m_attack;
    DefenseType m_defense;
};

struct restCommand : BasicCommand
{
    restCommand()
    {
        m_command = cmdRest;
    }
};

struct moveForwardCommand : BasicCommand
{
    moveForwardCommand()
    {
        m_command = cmdMoveForward;
    }
};

struct moveBackwardCommand : BasicCommand
{
    moveBackwardCommand()
    {
        m_command = cmdMoveBackward;
    }
};

struct turnLeftCommand : BasicCommand
{
    turnLeftCommand()
    {
        m_command = cmdTurnLeft;
    }
};

struct turnRightCommand : BasicCommand
{
    turnRightCommand()
    {
        m_command = cmdTurnRight;
    }
};

struct eatCommand : BasicCommand
{
    eatCommand()
    {
        m_command = cmdEat;
    }
};

struct fightCommand : BasicCommand
{
    fightCommand()
    {
        m_command = cmdFight;
    }
};
