#include "d3dcamera.h"

D3DCamera::D3DCamera(void)
: m_vEye(5.0f, 5.0f, -5.0f)
, m_vLook(-0.5f, -0.5f, 0.5f)
, m_vUp(0.0f, 1.0f, 0.0f)
, m_vRight(1.0f, 0.0f, 0.0f)
{
}

D3DCamera::~D3DCamera(void)
{
}

bool D3DCamera::create(IDirect3DDevice8 * pd3dDevice)
{
    if( NULL == pd3dDevice )
    {
        return( false );
    }

    m_pd3dDevice = pd3dDevice;

    return( true );
}

bool D3DCamera::updateView(void)
{
    if( NULL == m_pd3dDevice )
    {
        return( false );
    }

    D3DXMATRIX view;
    D3DXMatrixIdentity( &view );

    D3DXVec3Normalize( &m_vLook, &m_vLook );
    D3DXVec3Cross( &m_vRight, &m_vUp, &m_vLook );
    D3DXVec3Normalize( &m_vRight, &m_vRight );
    D3DXVec3Cross( &m_vUp, &m_vLook, &m_vRight );
    D3DXVec3Normalize( &m_vUp, &m_vUp );

    view._11 = m_vRight.x;
    view._12 = m_vUp.x;
    view._13 = m_vLook.x;
    view._14 = 0.0f;

    view._21 = m_vRight.y;
    view._22 = m_vUp.y;
    view._23 = m_vLook.y;
    view._24 = 0.0f;

    view._31 = m_vRight.z;
    view._32 = m_vUp.z;
    view._33 = m_vLook.z;
    view._34 = 0.0f;

    view._41 = -D3DXVec3Dot( &m_vEye, &m_vRight );
    view._42 = -D3DXVec3Dot( &m_vEye, &m_vUp );
    view._43 = -D3DXVec3Dot( &m_vEye, &m_vLook );
    view._44 =  1.0f;

    m_pd3dDevice->SetTransform( D3DTS_VIEW, &view );

    return( true );
}
